function [G,VL,x,VM,F] = MakeSubgradient(Mu,V,Paths,FlowUsage, MaxAcc, ZERO_THRESH)

VMu = V*Mu;
I = find(VMu>=max(VMu)-MaxAcc*max(VMu)); % find the active assignments - if direction is not a descent, then maybe this set can be changed
U=zeros(length(Paths),1);
for i=1:length(Paths)
    U(i) = 1/sum(Mu(Paths{i})); % find flows
end
F=zeros(size(V,2),1);
for i=1:size(V,2)
    F(i) = sum(U(FlowUsage{i}));    % link rates required by the flows
end

if (length(I)+sum(Mu<=ZERO_THRESH)>1)
    %sub differential has more than 1 element
    % find the subgradient that is nearest to the zero vector
    % this is the same as finding the schedule that is of assignments that most closely provides the required link bit-rates
    % - but why not find a schedule that provides more bit-rate than required - this can be solves with linear programming - whether this is possible requires further work
    if (min(Mu)>ZERO_THRESH)
        V2 = V(I,:);    
        VM=V2;
        VV = zeros(length(I),size(V,2));
            for i=1:length(I)
                VV(i,:) = F'-V(I(i),:);
            end
        if (1)            
            x = (VV*VV') \ ones(size(VV,1),1);
            if (max(isnan(x))==0 && max(x)<inf && min(x)>-inf && (max(abs(VV*VV'*x - ones(size(VV,1),1)))<1e-12 || min(x)>0))
                x = x/sum(x);
                G = VV'*x;
                VL = VV;
                return;
            end           
        end
         %disp('simple inverse failed');

        
        LastX = [1;zeros(size(V2,1)-1,1)];
        options = optimset('LargeScale','off','Display','off');
      %  x = quadprog(V2*V2',-F'*V2',[],[],ones(1,size(V2,1)),1,zeros(1,size(V2,1)),inf*ones(1,size(V2,1)),LastX,options);
        x = quadprog(VV*VV',[],[],[],ones(1,size(V2,1)),1,zeros(1,size(V2,1)),inf*ones(1,size(V2,1)),LastX,options);
        
        x = max(0,x);        
        G = F-(x'*V2)';
        for i = 1 : size(V2,1)
            VL(i,:) = F'-V2(i,:);
        end
    else      
        V2 = V(I,:);        
        for i=1:length(I)
            VV(i,:) = (F'-V(I(i),:));
        end
         J = find(Mu<=ZERO_THRESH);
         II = find(min(VV(:,J),[],2)<0); % the set of subgradients that have negative elements where Mu is 0
         NotII = find(min(VV(:,J),[],2)>=0); % the set of subgradients that have non-negative elements where Mu is 0
         VL = VV(NotII,:);
         for ii = 1:length(II)
             VL = [VL;MakeAllProjections(Mu,VV(II(ii),:),ZERO_THRESH)];
         end
         VM=0*VL;
         for i=1:size(VL,1)
             VM(i,:) = -VL(i,:) + F';
         end
         options = optimset('LargeScale','off','Display','off');
         Q = VL*VL';
         Name = 'QP Example';  
         c = [ ];
         A = ones(1,size(Q,1));
         b_L = [1]'; % Lower bounds on the linear constraints
         b_U = [1]'; % Upper bounds on the linear constraints
         x_L = zeros(size(Q,1),1); % Lower bounds on the variables
         x_U = x_L+inf; % Upper bounds on the variables
         x_0 = [1;zeros(size(Q,1)-1,1)]; % Starting point
         % Assign routine for defining a QP problem.
         Prob = qpAssign(Q, c, A, b_L, b_U, x_L, x_U, x_0, Name,...
             [], [], [],[],[]);
         Result = tomRun('qp-minos', Prob, 0);         
         G = VL'*Result.x_k;         
         G(find(Mu<=ZERO_THRESH & G<0)) = 0;
         x=Result.x_k;
         if (min(VL*G)<0 && norm(VL*G)<1e-4)
             Q = 1000*Q;
             Name = 'QP Example';  
             c = [ ];
             A = ones(1,size(Q,1));
             b_L = [1]'; % Lower bounds on the linear constraints
             b_U = [1]'; % Upper bounds on the linear constraints
             x_L = zeros(size(Q,1),1); % Lower bounds on the variables
             x_U = x_L+inf; % Upper bounds on the variables
             x_0 = x; % Starting point
             % Assign routine for defining a QP problem.
             Prob = qpAssign(Q, c, A, b_L, b_U, x_L, x_U, x_0, Name,...
                 [], [], [],[],[]);
             Result = tomRun('qp-minos', Prob, 0);         
             G1 = VL'*Result.x_k;         
             G1(find(Mu<=0 & G1<0)) = 0;
             x1=Result.x_k;
             if (norm(VL*G1)<norm(VL*G)) % select the one with the smaller derivative
                 G = G1;
                 x = x1;
             end
         end             
    end         
     
    
    

 else
     % only one gradient - so steepest decent is in that direction
     VL = F' - V(I,:); % negative of grad
     VM=V(I,:);
     G = VL'; % descent direction
     x=1;
 end

